// globals ----------------------------------      
var canReadTimer;
var connectState = 0;       /* valid values are: 0 = not connected; 1 = connecting; 2 = connected with channel handle */
var isBusOn = false;
var rxCount = 0;
var txCount = 0;
var errCount = 0;
var remotePort = 8080;
var remoteDevice = "192.168.0.132";
var kvaserSessionId = 0;
var globalHnd = -666;
var globalPollTime = 500;
var sessionTimeout = 20;// in seconds
var defaultPwd = "";

// callback identifiers:
var initializelib_reply = 0;
var opench_reply = 1;
var read_reply = 2;
var write_reply = 3;
var buson_reply = 4;
var busparam_reply = 5;
var drivertype_reply = 6;
var busoff_reply = 7;
var close_reply = 8;
var unloadlib_reply = 9;


// ---------------------------------------------------------------------------
//    min(x, y)
//
//    Function returns the smaller of the two parameters provided.  If the 
//    parameters are equal, y is returned.
// ---------------------------------------------------------------------------
function min(x, y) {
   if (x < y) return x;
   else return y;
}  // min()


// ---------------------------------------------------------------------------
//    padLeft(s, width, space)
//
//    This function pads the provided string s on the left until a string 
//    length of width is reached.  The padding character is determined by
//    space.  If space is true, ' ' is used.  If space is false, '0' is used.
//    The routine has a maximum width limit of 20.
// ---------------------------------------------------------------------------
function padLeft(s, width, space) {
   var padSize = width - s.length;
   var padding;

   if ((width > 20) || (padSize == 0)) return s;
   if (space)
      padding = "                    ";
   else
      padding = "00000000000000000000";

   return (padding.substr(0, padSize) + s);
}  // padLeft()


// ---------------------------------------------------------------------------
//    translateFlags(flags)
//
//    This function returns a three character length string that represents
//    the flags value.  The string will contain some combination of X, R, and
//    O.  X means the message was extended id.  R means the message was a
//    remote frame request.  O means an overrun has occurred since the last
//    read.
// ---------------------------------------------------------------------------
function translateFlags(flags) {
   var strFlags;

   if (flags & 0x0004)
      strFlags = "X";
   else
      strFlags = " ";

   if (flags & 0x0001)
      strFlags += "R";
   else
      strFlags += " ";

   if (flags & 0x0600)
      strFlags += "O";
   else
      strFlags += " ";

   return (strFlags);
}  // translateFlags()


// ---------------------------------------------------------------------------
//    readCookie(cName)
//
//    Function returns the value stored in the cookie for the provided name.
//    If the name is not found in the cookie, the function returns a value
//    of null.
// ---------------------------------------------------------------------------
function readCookie(cName) {
   var cValue = document.cookie;
   var cStart = cValue.indexOf(" " + cName + "=");

   if (cStart == -1) {
      cStart = cValue.indexOf(cName + "=");
   }
   if (cStart == -1) {
      cValue = null;
   }
   else {
      cStart = cValue.indexOf("=", cStart) + 1;
      var cEnd = cValue.indexOf(";", cStart);
      if (cEnd == -1) {
         cEnd = cValue.length;
      }
      cValue = unescape(cValue.substring(cStart, cEnd));
   }
   // return cValue;
   if (cValue != null && cValue != "") {
      console.log("Read cookie(" + cName + "): " + cValue);
      return cValue;
   }
   else return null;
}  // readCookie()


// ---------------------------------------------------------------------------
//    setCookie(cName, value, exdays)
//
//    Store the provided data in a cookie for retrieval later.  This is 
//    necessary to make sure the connection is destroyed when the application
//    is run again.  Otherwise, a session may still be in use from the last
//    time the browser ran the page.
// ---------------------------------------------------------------------------
function setCookie(cName, value, exdays) {
   var exdate = new Date();
   exdate.setDate(exdate.getDate() + exdays);
   var cValue = escape(value) + ((exdays == null) ? "" : "; expires=" + exdate.toUTCString());
   document.cookie = cName + "=" + cValue;
}  // setCookie()


// ---------------------------------------------------------------------------
//    canUnloadLibrary()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canUnloadLibrary function.  This releases the Blackbird for use by 
//    other computers by closing the session.
// ---------------------------------------------------------------------------
function canUnloadLibrary() {
   var func = "canUnloadLibrary";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { ident: unloadlib_reply }, function () { });
}  // canUnloadLibrary()


// ---------------------------------------------------------------------------
//    canClose()
//
//    Builds the url and sends it to the Blackbird to execute the canClose
//    function.  This closes the handle being used by the computer.
// ---------------------------------------------------------------------------
function canClose() {
   var func = "canClose";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { ident: close_reply, hnd: globalHnd }, function () { });
}  // canClose()


// ---------------------------------------------------------------------------
//    canOpenChannel()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canOpenChannel function.  This allows for specifying the channel to be
//    used and channel specific behavior.
// ---------------------------------------------------------------------------
function canOpenChannel() {
   var func = "canOpenChannel";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { ident: opench_reply, channel: 0, flags: 8 }, function () { });
}  // canOpenChannel()


// ---------------------------------------------------------------------------
//    canInitializeLibrary()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canInitializeLibrary function.  This call is necessary to establish a
//    session with the Blackbird before any other commands can be sent to the
//    unit.
// ---------------------------------------------------------------------------
function canInitializeLibrary() {
   var func = "canInitializeLibrary";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { mode: "jsonp", password: defaultPwd, ident: initializelib_reply, timeout: sessionTimeout }, function () { });
}  // canInitializeLibrary()


// ---------------------------------------------------------------------------
//    canSetBusParams()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canSetBusParams function.  This will set the bit rate based on the 
//    bitrateSel control on the webpage.  The options must equate to one of
//    the preset bit rate definitions.
// ---------------------------------------------------------------------------
function canSetBusParams() {
   var func = "canSetBusParams";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   var bitrateVal = $("select#bitrateSel").val()
   $.getJSON(url + "?callback=?", { freq: bitrateVal, ident: busparam_reply, hnd: globalHnd }, function () { });
}  // canSetBusParams


// ---------------------------------------------------------------------------
//    canRead()
//
//    Builds the url and sends it to the Blackbird to execute the canRead
//    function.  This will retrieve messages from the Blackbird receive 
//    buffer.
//
//    NOTE: The callback that is called when canRead() is finished calls 
//          canRead() again until it returns -2 (nomsg)
// ---------------------------------------------------------------------------
function canRead() {
   var func = "canRead";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { ident: read_reply, hnd: globalHnd, max: 200 }, function () { });
}  // canRead()


// ---------------------------------------------------------------------------
//    canSetBusOutputControl()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canSetBusOutputControl function.  This sets whether the Blackbird is in
//    normal or silent mode operation.
// ---------------------------------------------------------------------------
function canSetBusOutputControl() {
   var func = "canSetBusOutputControl";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;

   // A value of 4 means the driver mode is normal.  A value of 1 means silent mode.
   var driverMode = (($("silentMode").val() == 1) ? 1 : 4);
   $.getJSON(url + "?callback=?", { drivertype: driverMode, ident: drivertype_reply, hnd: globalHnd }, function () { });
}  // canSetBusOutputControl()

// ---------------------------------------------------------------------------
//    canBusOn()
//
//    Builds the url and sends it to the Blackbird to execute the canBusOn 
//    function.  This makes the Blackbird channel active on the bus so it 
//    can now receive and transmit CAN frames.
// ---------------------------------------------------------------------------
function canBusOn() {
   var func = "canBusOn";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { ident: buson_reply, hnd: globalHnd }, function () { });
}  // canBusOn()


// ---------------------------------------------------------------------------
//    canBusOff()
//
//    Builds the url and sends it to the Blackbird to execute the canBusOff 
//    function.  This deactivates the Blackbird channel so it is no longer 
//    receiving or transmitting CAN frames.
// ---------------------------------------------------------------------------
function canBusOff() {
   var func = "canBusOff";
   console.log(func);
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   $.getJSON(url + "?callback=?", { ident: busoff_reply, hnd: globalHnd }, function () { });
}  // canBusOff()


// ---------------------------------------------------------------------------
//    canWrite()
//
//    Builds the url and sends it to the Blackbird to execute the canWrite 
//    function.  This places the provided CAN frame in the transmit buffer
//    of the Blackbird V2 for transmission.
// ---------------------------------------------------------------------------
function canWrite() {

   var func = "canWrite";
   var url = "http://" + remoteDevice + kvaserSessionId + "/" + func;
   console.log(func);

   // Using parseInt allows the number to be entered in decimal, octal, or hexadecimal
   var canid = parseInt($('#canId').val()).toString();
   var candlc = parseInt($('#canDlc').val()).toString();

   // convert the flags input from an x/X for extended id r/R for remote frame request
   // to the appropriate numeric value combination
   var canflag = 2;
   var upperFlag = $('#canFlag').val().toUpperCase();
   if (upperFlag.search("X") != -1) {
      canflag = 4;
   }
   if (upperFlag.search("R") != -1) {
      canflag |= 1;
   }
   var byte0 = parseInt($('#byte0').val()).toString();
   var byte1 = parseInt($('#byte1').val()).toString();
   var byte2 = parseInt($('#byte2').val()).toString();
   var byte3 = parseInt($('#byte3').val()).toString();
   var byte4 = parseInt($('#byte4').val()).toString();
   var byte5 = parseInt($('#byte5').val()).toString();
   var byte6 = parseInt($('#byte6').val()).toString();
   var byte7 = parseInt($('#byte7').val()).toString();
   var canmsg = byte0 + "," + byte1 + "," + byte2 + "," + byte3 + "," +
                byte4 + "," + byte5 + "," + byte6 + "," + byte7;

   $.getJSON(url + "?callback=?", {id: canid, dlc: candlc, flag: canflag, msg: canmsg,
                                   ident: write_reply, hnd: globalHnd}, function () { });

} // canWrite


// ---------------------------------------------------------------------------
//    initialize()
//
//    Function tries to create a session after making sure that any previous
//    session is closed.
// ---------------------------------------------------------------------------
function initialize() {
   console.log("Initialize()");

   console.log("Trying to recover...");

   // read device to work with...
   remoteDevice = $("#ipAddr").val() + ":" + remotePort + "/";
   console.log(remoteDevice);
   alert(remoteDevice);
   var oldSession = readCookie('session');
   kvaserSessionId = oldSession;

   var oldHnd = readCookie('hnd');
   globalHnd = oldHnd;

   // Close any session that was left open
   if (globalHnd != -1) {
      canClose();
      globalHnd = -1;
      canUnloadLibrary();
   }

   // Try to open a session with the Blackbird
   kvaserSessionId = "";
   canInitializeLibrary();
}  // initialize()


// ---------------------------------------------------------------------------
//    canlib_callback(x)
//
//    This function is called whenever a json(p)-response is received from 
//    the Blackbird.
// ---------------------------------------------------------------------------
function canlib_callback(x) {
   var i = 0;
   var j = 0;

   switch (x.ident) {
      case read_reply:                                // canRead
         if (x.stat == 0) {
            if (typeof x.msgs != 'undefined') {
               for (i in x.msgs) {
                  var traceText;

                  if (x.msgs[i].flag & 0x0020) {
                     traceText = "   ErrFrame                                          ";
                     errCount++;
                  }
                  else {
                     traceText = "   " + padLeft(x.msgs[i].id.toString(16), 8, true) + "     ";
                     traceText += translateFlags(x.msgs[i].flag) + "     " + x.msgs[i].dlc.toString(16) + "    ";
                     for (j = 0; j < min(x.msgs[i].dlc, 8) ; j++) {
                        traceText += padLeft(x.msgs[i].msg[j].toString(16), 2, false) + " ";
                     }
                     traceText += "                                ".substr(0, (8 - min(x.msgs[i].dlc, 8)) * 3);
                     rxCount++;
                  }
                  traceText += "  " + (x.msgs[i].time / 1000000).toFixed(6) + "<br>";
                  $("#canTrace").prepend(traceText);
               } // end of i in x.msgs loop

               $("#errTotal").html(errCount);
               $("#rxTotal").html(rxCount);

               canRead(); // we call canRead again in case there are more messages. 

            }  // end of valid msgs structure returned
         }// end of canRead returned a message
         else if (x.stat == -2 /* nomsg */) {
         }
         else { // some other error occurred 
            console.log("stat:" + x.stat);            
         }

         break;
      case initializelib_reply:
         console.log(JSON.stringify(x));
         kvaserSessionId = x.session;
         setCookie("session", kvaserSessionId, 1);
         connectState = 2;
         $("#connect").button('complete');
         break;
      case opench_reply:
         if (x.hnd >= 0) {
            globalHnd = x.hnd;
            setCookie("hnd", globalHnd, 1);
            canSetBusOutputControl();
         }
         else {
            console.log("canOpenChannel() FAILED: " + JSON.stringify(x));
            connectState = 0;
            $("#busOn").button('reset');
            $("#connect").button('reset');
         }
         break;
      case drivertype_reply:
         if (x.stat == 0) {
            canSetBusParams();
         }
         else {
            console.log(JSON.stringify(x));
            canClose();
         }
         break;
      case busparam_reply:
         if (x.stat == 0) {
            canBusOn();
         }
         else {
            console.log(JSON.stringify(x));
            canClose();
         }
         break;
      case buson_reply:
         if (x.stat == 0) {
            // Change the button state to reflect fully connected
            $("#busOn").button('complete');
            isBusOn = true;

            // When we get the buson_reply, we start a periodic timer that tries
            // to read can messages. 
            canReadTimer = setInterval("canRead();", globalPollTime);
         }
         else {
            console.log(JSON.stringify(x));
            canClose();
         }
         break;
      case busoff_reply:
         if (x.stat != 0) {
            console.log(JSON.stringify(x));
         }
         isBusOn = false;
         $("#busOn").button('reset');
         canClose();
         break;
      case write_reply:
         if (x.stat == 0) {
            // increment transmit count
            txCount++;
            $("#txTotal").html(txCount);
         }
         else {
            console.log(JSON.stringify(x));
         }
         break;
      case close_reply:
         if (x.stat == 0) {
            globalHnd = -1;
         }
         else console.log(JSON.stringify(x));
         break;
      default:
         console.log(JSON.stringify(x));
         break;
   } // switch
}  // canlib_callback()

// ---------------------------------------------------------------------------
//    ClearTrace()
//
//    This function is activated by the Clear button in the CAN frame
//    display well.  When called the collected message data is cleared from
//    the string.
// ---------------------------------------------------------------------------
function ClearTrace() {
   $("#canTrace").empty();
}  // ClearTrace()

// ---------------------------------------------------------------------------
//    OnConnect()
//
//    This routine is triggered by the Connect/Disconnect button being pressed
//    on the web page.  This routine determines the current state of the
//    connection with the Blackbird (ie. the button state) and then either
//    starts the connection process or stops the connection process.
// ---------------------------------------------------------------------------
function OnConnect() {
   var func;

   switch (connectState) {
      case 0:
         ClearTrace();    // empty the output window from previous recording
         $("#connect").button('loading');
         connectState = 1;
         initialize();
         break;
      case 1:
         // do nothing.  We are still trying to process the last connect press
         break;
      case 2:
         // stop calling canRead() periodically
         clearInterval(canReadTimer);
         connectState = 0;
         $("#connect").button('reset');

         if (isBusOn) {
            canBusOff();
         }
         break;
   }
}  // OnConnect()

// ---------------------------------------------------------------------------
//    OnBusOn()
//
//    This routine is triggered by the Go Bus On/Off button being pressed
//    on the web page.  This routine determines the current state of the
//    connection with the Blackbird (connected or not) and then either
//    makes the channel active on the bus or de-activates the channel.
// ---------------------------------------------------------------------------
function OnBusOn() {
   if (connectState == 2) {
      if (!isBusOn) {
         ClearTrace();
         rxCount = 0;
         txCount = 0;
         errCount = 0;
         $("#txTotal").html(txCount);
         $("#rxTotal").html(rxCount);
         $("#errTotal").html(errCount);

         canOpenChannel(); // get handle
      }
      else {

         // stop calling canRead() periodically
         clearInterval(canReadTimer);

         canBusOff();
      } 
   } // if connected
   else {
      alert("You must be connected to the unit first.");
   }
} // OnBusOn()