// globals ----------------------------------      
var canread_timer;      
var busOnState         = 0;       /* valid values are: 0 = bus off; 1 = connecting; 2 = bus on */
var remote_port        = 8080;
var remote_device      = "kv-06348-000121"; 
var kvaser_session_id  = 0;
var global_hnd         = -666;
var global_poll_time   = 500;
var session_timeout    = 20;// in seconds
var default_pwd        = ""; 
     
// callback identifiers:
var initializelib_reply = 0; 
var opench_reply        = 1;
var read_reply          = 2;
var write_reply         = 3;
var buson_reply         = 4;
var busparam_reply      = 5;
var drivertype_reply    = 6;
var busoff_reply        = 7;
var close_reply         = 8;
var unloadlib_reply     = 9;

       
// ---------------------------------------------------------------------------
//    min(x, y)
//
//    Function returns the smaller of the two parameters provided.  If the 
//    parameters are equal, y is returned.
// ---------------------------------------------------------------------------
function min(x, y) {
   if (x < y) return x;
   else return y;
}  // min()

      
// ---------------------------------------------------------------------------
//    padLeft(s, width, space)
//
//    This function pads the provided string s on the left until a string 
//    length of width is reached.  The padding character is determined by
//    space.  If space is true, ' ' is used.  If space is false, '0' is used.
//    The routine has a maximum width limit of 20.
// ---------------------------------------------------------------------------
function padLeft(s, width, space) {
   var padSize = width - s.length;
   var padding;

   if ((width > 20) || (padSize == 0)) return s;
   if (space)
      padding = "                    ";
   else
      padding = "00000000000000000000";

   return (padding.substr(0, padSize) + s);
}  // padLeft()


// ---------------------------------------------------------------------------
//    translateFlags(flags)
//
//    This function returns a three character length string that represents
//    the flags value.  The string will contain some combination of X, R, and
//    O.  X means the message was extended id.  R means the message was a
//    remote frame request.  O means an overrun has occurred since the last
//    read.
// ---------------------------------------------------------------------------
function translateFlags(flags) {
   var strFlags;

   if (flags & 0x0004)
      strFlags = "X";
   else
      strFlags = " ";

   if (flags & 0x0001)
      strFlags += "R";
   else
      strFlags += " ";

   if (flags & 0x0600)
      strFlags += "O";
   else
      strFlags += " ";

   return (strFlags);
}  // translateFlags()


// ---------------------------------------------------------------------------
//    readCookie(c_name)
//
//    Function returns the value stored in the cookie for the provided name.
//    If the name is not found in the cookie, the function returns a value
//    of null.
// ---------------------------------------------------------------------------
function readCookie(c_name) {
   var c_value = document.cookie;
   var c_start = c_value.indexOf(" " + c_name + "=");
        
   if (c_start == -1) {
      c_start = c_value.indexOf(c_name + "=");
   }
   if (c_start == -1) {
      c_value = null;
   }
   else {
      c_start = c_value.indexOf("=", c_start) + 1;
      var c_end = c_value.indexOf(";", c_start);
      if (c_end == -1) {
         c_end = c_value.length;
      }
      c_value = unescape(c_value.substring(c_start,c_end));
   }
   // return c_value;
   if (c_value != null && c_value != "") {
      console.log("Read cookie(" + c_name + "): "+ c_value);
      return c_value;
   }
   else return null;
}  // readCookie()


// ---------------------------------------------------------------------------
//    setCookie(c_name, value, exdays)
//
//    Store the provided data in a cookie for retrieval later.  This is 
//    necessary to make sure the connection is destroyed when the application
//    is run again.  Otherwise, a session may still be in use from the last
//    time the browser ran the page.
// ---------------------------------------------------------------------------
function setCookie(c_name, value, exdays) {
   var exdate=new Date();
   exdate.setDate(exdate.getDate() + exdays);
   var c_value=escape(value) + ((exdays==null) ? "" : "; expires=" + exdate.toUTCString());
   document.cookie=c_name + "=" + c_value;
}  // setCookie()

     
// ---------------------------------------------------------------------------
//    canUnloadLibrary()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canUnloadLibrary function.  This releases the Blackbird for use by 
//    other computers by closing the session.
// ---------------------------------------------------------------------------
function canUnloadLibrary() {
   var func = "canUnloadLibrary";
   console.log(func);
   var url =  "http://" + remote_device + kvaser_session_id+"/"+func;            
   $.getJSON(url + "?callback=?", {ident: unloadlib_reply}, function() {});                   
}  // canUnloadLibrary()


// ---------------------------------------------------------------------------
//    canClose()
//
//    Builds the url and sends it to the Blackbird to execute the canClose
//    function.  This closes the handle being used by the computer.
// ---------------------------------------------------------------------------
function canClose() {
   var func = "canClose";
   console.log(func);
   var url =  "http://" + remote_device + kvaser_session_id+"/"+func;            
   $.getJSON(url + "?callback=?", {ident: close_reply, hnd:global_hnd}, function() {});             
}  // canClose()


// ---------------------------------------------------------------------------
//    canOpenChannel()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canOpenChannel function.  This allows for specifying the channel to be
//    used and channel specific behavior.
// ---------------------------------------------------------------------------
function canOpenChannel() {
   var func = "canOpenChannel";
   console.log(func);
   var url =  "http://" + remote_device + kvaser_session_id+"/"+func;            
   $.getJSON(url + "?callback=?", {ident: opench_reply, channel:0, flags:8}, function() {});           
}  // canOpenChannel()


// ---------------------------------------------------------------------------
//    canInitializeLibrary()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canInitializeLibrary function.  This call is necessary to establish a
//    session with the Blackbird before any other commands can be sent to the
//    unit.
// ---------------------------------------------------------------------------
function canInitializeLibrary() {
   var func = "canInitializeLibrary";
   console.log(func);
   var url =  "http://" + remote_device + kvaser_session_id+"/"+func;            
   $.getJSON(url + "?callback=?", {mode:"jsonp", password: default_pwd, ident:initializelib_reply, timeout: session_timeout}, function() {}); 
}  // canInitializeLibrary()


// ---------------------------------------------------------------------------
//    canSetBusParams()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canSetBusParams function.  This will set the bit rate based on the 
//    bitrate_sel control on the webpage.  The options must equate to one of
//    the preset bit rate definitions.
// ---------------------------------------------------------------------------
function canSetBusParams() {
   var func = "canSetBusParams";
   console.log(func);
   var url = "http://" + remote_device + kvaser_session_id + "/" + func;
   var bitrate_val = $("select#bitrate_sel").val()
   $.getJSON(url + "?callback=?", { freq: bitrate_val, ident: busparam_reply, hnd: global_hnd }, function () { });
}  // canSetBusParams


// ---------------------------------------------------------------------------
//    canRead()
//
//    Builds the url and sends it to the Blackbird to execute the canRead
//    function.  This will retrieve messages from the Blackbird receive 
//    buffer.
//
//    NOTE: The callback that is called when canRead() is finished calls 
//          canRead() again until it returns -2 (nomsg)
// ---------------------------------------------------------------------------
function canRead() {
   var func = "canRead";
   console.log(func);
   var url = "http://" + remote_device + kvaser_session_id + "/" + func;
   $.getJSON(url + "?callback=?", { ident: read_reply, hnd: global_hnd, max: 200 }, function () { });
}  // canRead()


// ---------------------------------------------------------------------------
//    canSetBusOutputControl()
//
//    Builds the url and sends it to the Blackbird to execute the 
//    canSetBusOutputControl function.  This sets whether the Blackbird is in
//    normal or silent mode operation.
// ---------------------------------------------------------------------------
function canSetBusOutputControl() {
   var func = "canSetBusOutputControl";
   console.log(func);
   var url = "http://" + remote_device + kvaser_session_id + "/" + func;

   // A value of 4 means the driver mode is normal.  A value of 1 means silent mode.
   var drivermode = 4;
   $.getJSON(url + "?callback=?", { drivertype: drivermode, ident: drivertype_reply, hnd: global_hnd }, function () { });
}  // canSetBusOutputControl()

// ---------------------------------------------------------------------------
//    canBusOn()
//
//    Builds the url and sends it to the Blackbird to execute the canBusOn 
//    function.  This makes the Blackbird channel active on the bus so it 
//    can now receive and transmit CAN frames.
// ---------------------------------------------------------------------------
function canBusOn() {
   var func = "canBusOn";
   console.log(func);
   var url = "http://" + remote_device + kvaser_session_id + "/" + func;
   $.getJSON(url + "?callback=?", { ident: buson_reply, hnd: global_hnd }, function () { });
}  // canBusOn()


// ---------------------------------------------------------------------------
//    canBusOff()
//
//    Builds the url and sends it to the Blackbird to execute the canBusOff 
//    function.  This deactivates the Blackbird channel so it is no longer 
//    receiving or transmitting CAN frames.
// ---------------------------------------------------------------------------
function canBusOff() {
   var func = "canBusOff";
   console.log(func);
   var url = "http://" + remote_device + kvaser_session_id + "/" + func;
   $.getJSON(url + "?callback=?", { ident: busoff_reply, hnd: global_hnd }, function () { });
}  // canBusOff()

     
// ---------------------------------------------------------------------------
//    initialize()
//
//    Function tries to create a session after making sure that any previous
//    session is closed.
// ---------------------------------------------------------------------------
function initialize() {
   console.log("Initialize()");
        
   console.log("Trying to recover...");
        
   // read device to work with...
   remote_device = $("#ipAddr").val() + ":" + remote_port + "/";
   console.log(remote_device);
   alert(remote_device);
   var oldsession = readCookie('session');
   kvaser_session_id = oldsession;        
        
   var oldhnd = readCookie('hnd');
   global_hnd = oldhnd;       
   
   // Close any session that was left open
   if (global_hnd != -1) {
      canClose();
      global_hnd = -1;
      canUnloadLibrary();
   }
        
   // Try to open a session with the Blackbird
   kvaser_session_id = "";        
   canInitializeLibrary();
}  // initialize()


// ---------------------------------------------------------------------------
//    canlib_callback(x)
//
//    This function is called whenever a json(p)-response is received from 
//    the Blackbird.
// ---------------------------------------------------------------------------
function canlib_callback(x)
{
   var i = 0;
   var j = 0;
   
   switch (x.ident) {
      case read_reply:                                // canRead
         if (x.stat == 0) {
            if (typeof x.msgs != 'undefined') {
               for(i in x.msgs) {
                  var trace_text
                
                  //console.log(JSON.stringify(x));
                  if (x.msgs[i].flag & 0x0020) {
                     trace_text = "Errorframe<br>";
                  }
                  else {
                     trace_text = " " + padLeft(x.msgs[i].id.toString(16), 8, true) + "      ";
                     trace_text += translateFlags(x.msgs[i].flag) + "      " + x.msgs[i].dlc.toString(16) + "     ";
                     for (j = 0; j < min(x.msgs[i].dlc,8); j++) {
                        trace_text += padLeft(x.msgs[i].msg[j].toString(16), 2, false) + "  ";
                     } 
                     trace_text += "                                ".substr(0, (8 - min(x.msgs[i].dlc, 8)) * 4);
                     trace_text += "  " + (x.msgs[i].time / 1000000).toFixed(6) + "<br>";
              
                  }
                  $("#can-trace").prepend(trace_text);
               }
               canRead(); // we call canRead again in case there are more messages. 
            }
            else if (x.stat == -2 /* nomsg */) {
            }
            else {
               //console.log("stat:" + x.stat);            
            }
         }
         break;
      case initializelib_reply:
         console.log(JSON.stringify(x));
         kvaser_session_id = x.session;
         setCookie("session", kvaser_session_id, 1);
         canOpenChannel();
         break;
      case opench_reply:
         if (x.hnd >= 0) {
            global_hnd = x.hnd;
            setCookie("hnd", global_hnd, 1);
            canSetBusOutputControl();
         }
         else {
            console.log("canOpenChannel() FAILED: " + JSON.stringify(x));
            busOnState = 0;
            $("#connect").button('reset');
         }
         break;
      case drivertype_reply:
         if (x.stat == 0) {
            canSetBusParams();
         }
         else {
            console.log(JSON.stringify(x));
            busOnState = 0;
            $("#connect").button('reset');
         }
         break;
      case busparam_reply:
         if (x.stat == 0) {
            canBusOn();
         }
         else {
            console.log(JSON.stringify(x));
            busOnState = 0;
            $("#connect").button('reset');
         }
         break;
      case buson_reply:
         if (x.stat == 0) {
            // Change the button state to reflect fully connected
            $("#connect").button('complete');
            busOnState = 2;

            // When we get the buson_reply, we start a periodic timer that tries
            // to read can messages. 
            canread_timer = setInterval("canRead();", global_poll_time);
         }
         else {
            console.log(JSON.stringify(x));
            busOnState = 0;
            $("#connect").button('reset');
         }
         break;
      case busoff_reply:
         if (x.stat == 0) {            
            canClose();
         }
         else console.log(JSON.stringify(x));          
         break;
      case close_reply:
         if (x.stat == 0) {
            // set the connect button state back to showing Connect
            $("#connect").button('reset');
            global_hnd = -1;
            busOnState = 0;
         }
         else console.log(JSON.stringify(x));          
         break;
      default:
         console.log(JSON.stringify(x));
         break;
   } // switch
}  // canlib_callback()

// ---------------------------------------------------------------------------
//    ClearTrace()
//
//    This function is activated by the Clear button in the CAN frame
//    display well.  When called the collected message data is cleared from
//    the string.
// ---------------------------------------------------------------------------
function ClearTrace() {
   $("#can-trace").empty();
}  // ClearTrace()

// ---------------------------------------------------------------------------
//    OnConnect()
//
//    This routine is triggered by the Connect/Disconnect button being pressed
//    on the web page.  This routine determines the current state of the
//    connection with the Blackbird (ie. the button state) and then either
//    starts the connection process or stops the connection process.
// ---------------------------------------------------------------------------
function OnConnect() {
   var func;

   switch (busOnState) {
      case 0:
         ClearTrace();    // empty the output window from previous recording
         $("#connect").button('loading');
         busOnState = 1;
         initialize();
         break;
      case 1:
         // do nothing.  We are still trying to process the last connect press
         break;
      case 2:
         // stop calling canRead() periodically
         clearInterval(canread_timer);
          
         canBusOff();
         break;
   }
          
}  // OnConnect()
